+++
date = "2019-10-24"
weight = 100

title = "How to Release Packages"

aliases = [
    "/guidelines/versioning.md",
    "/old-wiki/Guidelines/Versioning",
    "/user_joykim/howtoreleasepackages.md",
    "/old-wiki/User:Joykim/HowToReleasePackages",
    "/old-wiki/Guidelines/HowToReleasePackages"
]
+++

This document aims to explain how to release packages to Apertis
projects.

# Development workflow

Apertis currently deals with two types of packages

  - Packages imported from upstream distributions: they live under
    [gitlab.apertis.org/pkg/](https://gitlab.apertis.org/pkg/) and
    follow the
    [Guidelines/Gitlab-based_packaging_workflow]( {{< ref "gitlab-based_packaging_workflow.md" >}} )
  - Packages for which Apertis is the upstream: they live under
    [appfw](https://gitlab.apertis.org/appfw),
    [hmi](https://gitlab.apertis.org/hmi), etc.

# Writing commit messages suitable for generating `debian/changelog` with `gbp-dch`

Commit messages should follow the [recommended
style]( {{< ref "/guidelines/contribution_process.md#commit-message" >}} ).

In addition,
[gbp-dch](http://honk.sigxcpu.org/projects/git-buildpackage/manual-html/man.gbp.dch.html)
is a tool to take those commit messages and automatically turn them in
entries for `debian/changelog`.

With it you can use the following tags in your commit messages:

  - `Gbp-Dch: {Full,Short}`
      - `Short` will create a short changelog entry using the short
        commit message (ie. the first line of the commit message)
      - `Full` will include the long description in the generated
        changelog entry
  - `Apertis: {$TASK_URL,$TASK_ID}`, is translated to `(Apertis: Txxxx)`
    in `debian/changelog` for automated task status tracking
  - `Thanks: $SOMEONE` if you want to give kudos to someone

Running `gbp dch` will turn a commit message like:

<code>

    Update website link to demo images

    The website has new path for demo images.

    Gbp-Dch: Full
    Apertis: https://phabricator.apertis.org/T12345
    Thanks: Mariano for pointing out the issue

    Signed-off-by: Héctor Orón Martínez <hector.oron@collabora.co.uk>

</code>

to the following `debian/changelog` entry:

<code>

```
  * Update website link to demo images.
    The website has new path for demo images.
    Signed-off-by: Héctor Orón Martínez <hector.oron@collabora.co.uk>
    Thanks to Mariano for pointing out the issue (Apertis: T12345)
```

</code>

Just make sure your package has the following entries in
`debian/gbp.conf`:

<code>

    [dch]
    ignore-branch = True
    meta-closes = Apertis
    meta-closes-bugnum = (?:https://phabricator.apertis.org/)?(\T?\d+)

</code>

# Getting prepared for release

## Preparing a GPG key

While releasing, the maintainer should provide a GPG-signed version tag.
If you have a GPG key already, please skip this section. Otherwise, you
need to generate a new key by the following steps.

    $ gpg --gen-key

At the prompt, you need to specify several options by interactive mode.
In most cases, it provides the default values, but you can use the
desired values for each question. After generating a key pair, you can
use the command to list GPG keys.

    $ gpg --list-secret-keys --keyid-format LONG
    /Users/justin/.gnupg/secring.gpg
    --------------------------------
    sec   2048R/85AC7EE513BFB53B 2017-03-03
    uid                          Justin Kim <justin.kim@collabora.com>
    ssb   2048R/ADA53627F4F0C55C 2017-03-03

From the list, the GPG key ID is *85AC7EE513BFB53B*. Now you need to let
*git* know the key ID.

    $ git config --global user.signingkey "85AC7EE513BFB53B"

## Versioning

Module versioning differs for libraries and applications: libraries need
a libtool version specified in addition to their package version.
Applications just have a package version.

### Libtool versioning

Libraries have two version numbers: a libtool version which tracks ABI
backwards compatibility, and a [package
version](#package-versioning) which tracks feature changes.
These are normally incremented in synchronisation, but should be kept
separate because ABI backwards compatibility is not necessarily related
to feature changes or bug fixes. Furthermore, the two version numbers
have different semantics, and cannot be automatically generated from
each other.

A good overview of libtool versioning, and the differences from package
versioning, is given in the [Autotools
Mythbuster](https://autotools.io/libtool/version.html).

To update the libtool version, follow the algorithm given in the
comments below. This is a typical `configure.ac` snippet for setting up
libtool versioning:

    # Before making a release, the LT_VERSION string should be modified. The
    # string is of the form c:r:a. Follow these instructions sequentially:
    #   1. If the library source code has changed at all since the last update, then
    #      increment revision (‘c:r:a’ becomes ‘c:r+1:a’).
    #   2. If any interfaces have been added, removed, or changed since the last
    #      update, increment current, and set revision to 0.
    #   3. If any interfaces have been added since the last public release, then
    #      increment age.
    #   4. If any interfaces have been removed or changed since the last public
    #      release, then set age to 0.
    AC_SUBST([LT_VERSION],[0:0:0])

The following snippet can be used in a `Makefile.am` to pass that
version info to libtool:

    my_library_la_LDFLAGS = -version-info $(LT_VERSION)

If a package contains more than one library, it should usually have a
separate libtool version for each one.

The standard process for making a release of a module increments the
libtool version (if the module is a library) and the package version
immediately before release. This is called pre-release incrementing.

The use of pre-release increment for libtool versions means that they
are only incremented once for all ABI changes in a release. Some
projects use post-release increment for package versions to ensure that
the package version number is not outdated (i.e. still equal to the
previous release) during the development cycle; we achieve this by
altering the version number automatically in [the build-snapshot
script](https://gitlab.apertis.org/infrastructure/apertis-customizations/blob/apertis/v2020dev0/development/build-snapshot)
instead.

### Package versioning

The package version number for a library is that passed to `AC_INIT()`,
and the one which is typically known as the project’s version number.
For example, the Debian package for a library will use the library’s
package version.

Versions of packages developed for Apertis have the form `x.y.z` and are
updated according to the following rules:

  - `x` starts at 0, and increments when there is a major compatibility
    break. Libraries with different major versions should usually be
    [parallel-installable]( {{< ref "module_setup.md#parallel-installability" >}} ).
  - `y` is the Apertis branch year and month as a single 4-digit number,
    for instance `1706`.
  - `z` is 0 for the first release to each branch, and goes up for each
    release (whether it is a feature release or bugfix-only).

# Release process

The packages maintained in the [Apertis git
repositories](https://git.apertis.org) should be released with the
following procedure, roughly based on the [GNOME release
process](https://wiki.gnome.org/MaintainersCorner/Releasing). The
commands below assume you're running the
[Apertis SDK](https://designs.apertis.org/latest/software-development-kit.html).

1.  Make sure all the `git-buildpackage` local branches (`apertis/*`,
    `upstream/*` and `pristine-tar`) are available and up-to-date:
    <code>
        $ gbp pull
    </code>
2.  Choose the new version number by the rules of [Libtool
    versioning](#libtool-versioning) and [Package
    versioning](#package-versioning)
3.  Make sure you have no local changes and you are in right branch
    <code>
        $ git status
    </code>
4.  `configure.ac`: bump for a new upstream release, unless the module
    uses `git-version-gen` in which case you don't need to do anything.
5.  `debian/changelog`: [auto-generate the changelog
    entry](#generate-debian.2fchangelog-from-the-git-log-with-gbp-dch)
    and mark it for release replacing `UNRELEASED` with the current
    version (for instance `16.12`) <code>
        $ gbp dch --auto -R
    </code>
6.  Make sure that all tasks this release is fixing [are
    mentioned](https://designs.apertis.org/latest/contribution-process.html#automatically-closing-tasks)
    in `debian/changelog` in the form `(Apertis: T1234)`
7.  For a library: check that any `Since: UNRELEASED` annotations have
    been replaced with `Since: 0.1612.1` or whatever, for example using
    `git grep UNRELEASED`
8.  For a library: bump the Libtool current/revision/age versions in
    `configure.ac`
9.  Build test the package with
    [`build-snapshot`](https://gitlab.apertis.org/infrastructure/apertis-customizations/blob/apertis/v2020dev0/development/build-snapshot)
    <code>
        $ build-snapshot localhost
    </code>
10. For a library: look for output from `dpkg-makeshlibs` reporting new
    symbols that should be in the `.symbols` files. Add them to the
    `.symbols` files, with the upstream version (for example
    `mylib_foo@Base 0.1612.0`, not `mylib_foo@Base 0.1612.0-0co1`).
11. Now you need the following variables for the next steps. <code>
        PACKAGE=$(dpkg-parsechangelog -SSource)
        DEBVERSION=$(dpkg-parsechangelog -SVersion)
        VERSION=${DEBVERSION%%-*}
        RELEASE=$(dpkg-parsechangelog -SDistribution)
        COMPONENT=$(cat debian/source/apertis-component)
    </code> If `build-snapshot` succeeds, commit the changes. <code>
        $ git commit --all --signoff -m "Release $PACKAGE version $DEBVERSION"
    </code>
12. Submit a merge request for review following the [contribution
    process
    guidelines](https://designs.apertis.org/latest/contribution-process.html),
    paying special attention to the section about [release
    commits](https://designs.apertis.org/latest/contribution-process.html#release-commits)
    if you are releasing a new version of the package.
13. Skip this if you are only releasing changes under the `debian/`
    folder and not doing an upstream release at the same time:
    1.  If you're doing an upstream release, tag it (note the versioning
        must have a <strong>`v`</strong>, for instance `v1.2.3`). <code>
            $ git tag v$VERSION -s -m "Release $PACKAGE $VERSION"
        </code>
14. Tag the Apertis release. <code>
        $ gbp buildpackage --git-tag --git-tag-only --git-ignore-branch
    </code>
15. Run `build-snapshot` again. This is because the build procedure is
    slightly different with the tags.
16. Push the tags and push the changes (in that order so CI never builds
    these commits without the tags being present) <code>
        $ git push origin v$VERSION apertis/$VERSION
        $ git push
    </code>
17. If you happened to push the commits *before* the tags, you will need
    to manually trigger the corresponding CI job on Jenkins (or ask
    someone to do it for you)
18. Wait for an OBS maintainer to accept the request to merge the
    package built in the corresponding `apertis:$RELEASE:snapshots`
    project.

# Work done

Once accepted by OBS maintainer, you can update the package by `apt-get`
after a daily Apertis image is released.

# External links

  - [Autotools Mythbuster guide to library
    versioning](https://autotools.io/libtool/version.html)
  - [libtool manual page on version
    info](http://www.gnu.org/s/libtool/manual/html_node/Updating-version-info.html)
  - [GNOME release process
    instructions](https://wiki.gnome.org/MaintainersCorner/Releasing)
